@@ -157,12 +157,46 @@ def exit_cleanup():
 
 _rosdistro_index = None
 _rosdistro_distribution_files = {}
+_rosdistro_index_commit = None
+
+
+def get_index_url():
+    global _rosdistro_index_commit
+    index_url = rosdistro.get_index_url()
+    pr = urlparse(index_url)
+    if pr.netloc == 'raw.github.com':
+        # Try to determine what the commit hash was
+        tokens = [x for x in pr.path.split('/') if x]
+        if len(tokens) <= 3:
+            debug("Failed to get commit for rosdistro index file: index url")
+            debug(tokens)
+            return index_url
+        owner = tokens[0]
+        repo = tokens[1]
+        branch = tokens[2]
+        gh = get_github_interface(quiet=True)
+        if gh is None:
+            # Failed to get it with auth, try without auth (may fail)
+            gh = Github(username=None, auth=None)
+        try:
+            data = gh.get_branch(owner, repo, branch)
+        except GithubException:
+            debug(traceback.format_exc())
+            debug("Failed to get commit for rosdistro index file: api")
+            return index_url
+        _rosdistro_index_commit = data.get('commit', {}).get('sha', None)
+        if _rosdistro_index_commit is not None:
+            info("ROS Distro index file associate with commit '{0}'"
+                 .format(_rosdistro_index_commit))
+        else:
+            debug("Failed to get commit for rosdistro index file: json")
+    return index_url
 
 
 def get_index():
     global _rosdistro_index
     if _rosdistro_index is None:
-        _rosdistro_index = rosdistro.get_index(rosdistro.get_index_url())
+        _rosdistro_index = rosdistro.get_index(get_index_url())
         if _rosdistro_index.version == 1:
             error("This version of bloom does not support rosdistro version "
                   "'{0}', please use an older version of bloom."
@@ -195,6 +229,28 @@ def get_disitrbution_file_url(distro):
     return _rosdistro_distribution_file_urls[distro]
 
 
+def validate_github_url(url, url_type):
+    if 'github.com' not in url:
+        return True
+    valid_url = True
+    if not url.endswith('.git') and not url.endswith('.git/'):
+        valid_url = False
+        warning("The {0} repository url you provided does not end in `.git`."
+                .format(url_type))
+    if not url.startswith('https://'):
+        valid_url = False
+        warning("The {0} repository url you provided is not a `https://` address."
+                .format(url_type))
+    if not valid_url:
+        if maybe_continue(msg="Would you like to enter the address again"):
+            return False
+        else:
+            warning("Very well, the address '{0}' will be used as is.".format(url))
+            return True
+    # url is OK
+    return True
+
+
 def get_repo_uri(repository, distro):
     url = None
     # Fetch the distro file
@@ -213,7 +269,8 @@ def get_repo_uri(repository, distro):
              .format(repository, distro))
         info("You can continue the release process by manually specifying the location of the RELEASE repository.")
         info("To be clear this is the url of the RELEASE repository not the upstream repository.")
-        info("For release repositories on github, you should provide the `https://` url which should end in `.git`.")
+        info("For release repositories on GitHub, you should provide the `https://` url which should end in `.git`.")
+        info("Here is the url for a typical release repository on GitHub: https://github.com/ros-gbp/rviz-release.git")
         while True:
             try:
                 url = safe_input('Release repository url [press enter to abort]: ')
@@ -224,23 +281,11 @@ def get_repo_uri(repository, distro):
                 url = None
                 error("No release repository url given, aborting.", exit=True)
                 break
+            if url is None:
+                break
             # If github.com address, validate it
-            if url is not None and 'github.com' in url:
-                valid_url = True
-                if not url.endswith('.git') and not url.endswith('.git/'):
-                    valid_url = False
-                    warning("The release repository url you provided does not end in `.git`.")
-                if not url.startswith('https://'):
-                    valid_url = False
-                    warning("The release repository url you provided is not a `https://` address.")
-                if not valid_url:
-                    warning("Would you like to enter the address again?")
-                    if maybe_continue():
-                        url = None
-                        continue
-                    else:
-                        info("Very well, the address '{url}' will be used as is.".format(**locals()))
-                        break
+            if not validate_github_url(url, 'release'):
+                continue
             break
         global _user_provided_release_url
         _user_provided_release_url = url
@@ -332,7 +377,7 @@ def generate_ros_distro_diff(track, repository, distro):
             repo['packages'].remove(pkg_name)
     repo['packages'].sort()
 
-    def get_repository_info_from_user():
+    def get_repository_info_from_user(url_type):
         data = {}
         while True:
             vcs_type = safe_input('VCS type [git, svn, hg, bzr]: ')
@@ -345,13 +390,16 @@ def generate_ros_distro_diff(track, repository, distro):
         while True:
             url = safe_input('VCS url: ')
             if url:
+                if not validate_github_url(url, url_type):
+                    # User wants to try again
+                    continue
                 break
             error("Nothing entered for url.")
             if not maybe_continue(msg='Try again'):
                 return {}
         data['url'] = url
         while True:
-            version = safe_input('VCS version [commit, tag, branch, etc]: ')
+            version = safe_input('VCS version [commit, tag, or branch, e.g. master]: ')
             if version:
                 break
             error("Nothing entered for version.")
@@ -366,16 +414,16 @@ def generate_ros_distro_diff(track, repository, distro):
         if not docs and maybe_continue(msg='Would you like to add documentation information for this repository?'):
             info("Please enter your repository information for the doc generation job.")
             info("This information should point to the repository from which documentation should be generated.")
-            docs = get_repository_info_from_user()
+            docs = get_repository_info_from_user('doc')
         distribution_dict['repositories'][repository]['doc'] = docs
 
     # Ask for source entry
     if 'BLOOM_DONT_ASK_FOR_SOURCE' not in os.environ:
         source = distribution_dict['repositories'][repository].get('source', {})
         if not source and maybe_continue(msg='Would you like to add source information for this repository?'):
-            info("Please enter information which points ot the active development branch for this repository.")
+            info("Please enter information which points to the active development branch for this repository.")
             info("This information is used to run continuous integration jobs and for developers to checkout from.")
-            source = get_repository_info_from_user()
+            source = get_repository_info_from_user('source')
         distribution_dict['repositories'][repository]['source'] = source
 
     # Ask for maintainership information
@@ -387,7 +435,7 @@ def generate_ros_distro_diff(track, repository, distro):
             info("Valid maintenance statuses:")
             info("- developed: active development is in progress")
             info("- maintained: no new development, but bug fixes and pull requests are addressed")
-            info("- end-of-life: should not be used, will disapear at some point")
+            info("- end-of-life: should not be used, will disappear at some point")
             while True:
                 status = safe_input('Status: ')
                 if status in ['developed', 'maintained', 'end-of-life']:
@@ -450,7 +498,7 @@ def generate_ros_distro_diff(track, repository, distro):
                     continue
                 if repo not in distro_file_repos or distro_file_repos[repo] != distro_yaml['repositories'][repo]:
                     error("This generated pull request modifies a repository entry other than the one being released.")
-                    error("This likely occured because the upstream rosdistro changed during this release.")
+                    error("This likely occurred because the upstream rosdistro changed during this release.")
                     error("This pull request will abort, please re-run this command with the -p option to try again.",
                           exit=True)
         # Write the diff out to file
@@ -486,7 +534,13 @@ def create_fork(org, repo, user, password):
         error("Failed to create fork: {0} {1}".format(resp.status, resp.reason), exit=True)
 
 
-def get_github_interface():
+_gh = None
+
+
+def get_github_interface(quiet=False):
+    global _gh
+    if _gh is not None:
+        return _gh
     # First check to see if the oauth token is stored
     oauth_config_path = os.path.join(os.path.expanduser('~'), '.config', 'bloom')
     config = {}
@@ -499,22 +553,26 @@ def get_github_interface():
                 return Github(username, auth=auth_header_from_oauth_token(token), token=token)
     if not os.path.isdir(os.path.dirname(oauth_config_path)):
         os.makedirs(os.path.dirname(oauth_config_path))
+    if quiet:
+        return None
     # Ok, now we have to ask for the user name and pass word
     info("")
     warning("Looks like bloom doesn't have an oauth token for you yet.")
-    warning("Therefore bloom will require your Github username and password just this once.")
-    warning("With your Github username and password bloom will create an oauth token on your behalf.")
+    warning("Therefore bloom will require your GitHub username and password just this once.")
+    warning("With your GitHub username and password bloom will create an oauth token on your behalf.")
     warning("The token will be stored in `~/.config/bloom`.")
     warning("You can delete the token from that file to have a new token generated.")
     warning("Guard this token like a password, because it allows someone/something to act on your behalf.")
-    warning("If you need to unauthorize it, remove it from the 'Applications' menu in your Github account page.")
+    warning("If you need to unauthorize it, remove it from the 'Applications' menu in your GitHub account page.")
     info("")
+    if not maybe_continue('y', "Would you like to create an OAuth token now"):
+        return None
     token = None
     while token is None:
         try:
             username = getpass.getuser()
-            username = safe_input("Github username [{0}]: ".format(username)) or username
-            password = getpass.getpass("Github password (never stored): ")
+            username = safe_input("GitHub username [{0}]: ".format(username)) or username
+            password = getpass.getpass("GitHub password (never stored): ")
         except (KeyboardInterrupt, EOFError):
             return None
         if not password:
@@ -531,14 +589,26 @@ def get_github_interface():
         except GithubException as exc:
             error("{0}".format(exc))
             info("")
+            if '{0}'.format(exc.resp.status) in ['401']:
+                warning("Receiving 401 when trying to create an oauth token can be caused by the user "
+                        "having two-factor authentication enabled.")
+                warning("If 2FA is enabled, the user will have to create an oauth token manually.")
+                warning("A token can be created at https://github.com/settings/applications")
+                warning("The resulting token can be placed in the '{oauth_config_path}' file as such:"
+                        .format(**locals()))
+                info("")
+                warning('{{"github_user": "{username}", "oauth_token": "TOKEN_GOES_HERE"}}'
+                        .format(**locals()))
+                info("")
             warning("This sometimes fails when the username or password are incorrect, try again?")
             if not maybe_continue():
                 return None
+    _gh = gh
     return gh
 
 
 def get_changelog_summary(release_tag):
-    summary = ""
+    summary = u""
     packages = dict([(p.name, p) for p in get_packages().values()])
     for package_name in sorted(packages.keys()):
         package = packages[package_name]
@@ -555,23 +625,24 @@ def get_changelog_summary(release_tag):
                     for change in changes:
                         msgs.extend([i for i in to_unicode(change).splitlines()])
                     msg = '\n'.join(msgs)
-                    summary += """
+                    summary += u"""
 ## {package.name}
 """.format(**locals())
                     if msg:
-                        summary += """
+                        summary += u"""
 ```
 {msg}
 ```
 """.format(**locals())
                     else:
-                        summary += """
+                        summary += u"""
 - No changes
 """
     return summary
 
 
-def open_pull_request(track, repository, distro):
+def open_pull_request(track, repository, distro, interactive):
+    global _rosdistro_index_commit
     # Get the diff
     distribution_file = get_distribution_file(distro)
     if repository in distribution_file.repositories and \
@@ -592,6 +663,8 @@ def open_pull_request(track, repository, distro):
         return
     # Get the github interface
     gh = get_github_interface()
+    if gh is None:
+        return None
     # Determine the head org/repo for the pull request
     head_org = gh.username  # The head org will always be gh user
     head_repo = None
@@ -600,7 +673,7 @@ def open_pull_request(track, repository, distro):
         # If it is, then a fork is not necessary
         head_repo = base_repo
     else:
-        info(fmt("@{bf}@!==> @|@!Checking on github for a fork to make the pull request from..."))
+        info(fmt("@{bf}@!==> @|@!Checking on GitHub for a fork to make the pull request from..."))
         # It is not, so a fork will be required
         # Check if a fork already exists on the user's account with the same name
         base_full_name = '{base_org}/{base_repo}'.format(**locals())
@@ -627,7 +700,7 @@ def open_pull_request(track, repository, distro):
                     head_repo = repo['name']
         # If not head_repo still, a fork does not exist and must be created
         if head_repo is None:
-            warning("Could not find a fork of {base_full_name} on the {gh.username} Github account."
+            warning("Could not find a fork of {base_full_name} on the {gh.username} GitHub account."
                     .format(**locals()))
             warning("Would you like to create one now?")
             if not maybe_continue():
@@ -647,7 +720,7 @@ def open_pull_request(track, repository, distro):
     new_branch = None
     title = "{0}: {1} in '{2}' [bloom]".format(repository, version, base_path)
     track_dict = get_tracks_dict_raw()['tracks'][track]
-    body = """\
+    body = u"""\
 Increasing version of package(s) in repository `{repository}` to `{version}`:
 
 - upstream repository: {upstream_repo}
@@ -694,11 +767,13 @@ Increasing version of package(s) in repository `{repository}` to `{version}`:
                       "{base_org}/{base_repo}:{base_branch}".format(**locals()) +
                       "@|@!'?")
             info(msg)
-            if not maybe_continue():
+            if interactive and not maybe_continue():
                 warning("Skipping the pull request...")
                 return
             _my_run('git checkout -b {new_branch}'.format(**locals()))
             _my_run('git pull {rosdistro_url} {base_branch}'.format(**locals()), "Pulling latest rosdistro branch")
+            if _rosdistro_index_commit is not None:
+                _my_run('git reset --hard {_rosdistro_index_commit}'.format(**globals()))
             with open('{0}'.format(base_path), 'w') as f:
                 info(fmt("@{bf}@!==> @|@!Writing new distribution file: ") + str(base_path))
                 f.write(updated_distro_file_yaml)
@@ -951,6 +1026,15 @@ def perform_release(repository, track, distro, new_track, interactive, pretend,
                       str(tracks), exit=True)
             # Get the only track
             track = tracks[0]
+        # Make sure the release repository and the upstream repository are different.
+        track_dict = tracks_dict['tracks'][track]
+        vcs_uri = track_dict.get('vcs_uri')
+        if vcs_uri == release_repo.get_url():
+            warning("Your RELEASE repository, '{0}', is the same as your UPSTREAM repository, '{1}'."
+                    .format(release_repo.get_url(), vcs_uri))
+            warning("This is not recommended, normally you have separate RELEASE and UPSTREAM repositories.")
+            if not maybe_continue('n', 'Are you sure you want continue'):
+                error("User quit.", exit=True)
         start_summary(track)
         if not pull_request_only:
             _perform_release(repository, track, distro, new_track, interactive, pretend, tracks_dict)
@@ -959,7 +1043,7 @@ def perform_release(repository, track, distro, new_track, interactive, pretend,
              "Generating pull request to distro file located at '{0}'"
              .format(get_disitrbution_file_url(distro)))
         try:
-            pull_request_url = open_pull_request(track, repository, distro)
+            pull_request_url = open_pull_request(track, repository, distro, interactive)
             if pull_request_url:
                 info(fmt(_success) + "Pull request opened at: {0}".format(pull_request_url))
                 if 'BLOOM_NO_WEBBROWSER' not in os.environ and platform.system() in ['Darwin']:
