@@ -42,6 +42,7 @@ import sys
 import traceback
 
 from dateutil import tz
+from pkg_resources import parse_version
 
 from bloom.generators import BloomGenerator
 from bloom.generators import resolve_dependencies
@@ -310,6 +311,28 @@ def generate_substitutions_from_package(
             package.maintainers[0].name,
             package.maintainers[0].email
         ))
+    bad_changelog = False
+    # Make sure that the first change log is the version being released
+    if package.version != changelogs[0][0]:
+        error("")
+        error("The version of the first changelog entry '{0}' is not the "
+              "same as the version being currently released '{1}'."
+              .format(package.version, changelogs[0][0]))
+        bad_changelog = True
+    # Make sure that the current version is the latest in the changelog
+    for changelog in changelogs:
+        if parse_version(package.version) < parse_version(changelog[0]):
+            error("")
+            error("There is at least one changelog entry, '{0}', which has a "
+                  "newer version than the version of package '{1}' being released, '{2}'."
+                  .format(changelog[0], package.name, package.version))
+            bad_changelog = True
+    if bad_changelog:
+        error("This is almost certainly by mistake, you should really take a "
+              "look at the changelogs for the package you are releasing.")
+        error("")
+        if not maybe_continue('n', 'Continue anyways'):
+            sys.exit("User quit.")
     data['changelogs'] = changelogs
     # Use debhelper version 7 for oneric, otherwise 9
     data['debhelper_version'] = 7 if os_version in ['oneiric'] else 9
@@ -516,6 +539,25 @@ class DebianGenerator(BloomGenerator):
         update_rosdep()
         self.has_run_rosdep = True
 
+    def pre_modify(self):
+        info("\nPre-verifying Debian dependency keys...")
+        # Run rosdep update is needed
+        if not self.has_run_rosdep:
+            self.update_rosdep()
+
+        peer_packages = [p.name for p in self.packages.values()]
+
+        for package in self.packages.values():
+            depends = package.run_depends + package.buildtool_export_depends
+            build_depends = package.build_depends + package.buildtool_depends + package.test_depends
+            unresolved_keys = depends + build_depends + package.replaces + package.conflicts
+            for os_version in self.distros:
+                resolve_dependencies(unresolved_keys, self.os_name,
+                                     os_version, self.rosdistro,
+                                     peer_packages, fallback_resolver=missing_dep_resolver)
+
+        info("All keys are " + ansi('greenf') + "OK" + ansi('reset') + "\n")
+
     def pre_branch(self, destination, source):
         if destination in self.debian_branches:
             return
