@@ -43,6 +43,7 @@ import traceback
 import textwrap
 
 from dateutil import tz
+from time import strptime
 
 from bloom.generators import BloomGenerator
 from bloom.generators import resolve_dependencies
@@ -255,7 +256,9 @@ def generate_substitutions_from_package(
     data['Maintainers'] = ', '.join(maintainers)
     # Changelog
     if releaser_history:
-        sorted_releaser_history = sorted(releaser_history, key=releaser_history.get, reverse=True)
+        sorted_releaser_history = sorted(releaser_history,
+                                         key=lambda k: strptime(releaser_history.get(k)[0], '%a %b %d %Y'),
+                                         reverse=True)
         changelogs = [(v, releaser_history[v]) for v in sorted_releaser_history]
     else:
         # Ensure at least a minimal changelog
@@ -479,6 +482,29 @@ class RpmGenerator(BloomGenerator):
         update_rosdep()
         self.has_run_rosdep = True
 
+    def pre_modify(self):
+        info("\nPre-verifying RPM dependency keys...")
+        # Run rosdep update is needed
+        if not self.has_run_rosdep:
+            self.update_rosdep()
+
+        peer_packages = [p.name for p in self.packages.values()]
+
+        for package in self.packages.values():
+            depends = package.run_depends + package.buildtool_export_depends
+            build_depends = package.build_depends + package.buildtool_depends + package.test_depends
+            unresolved_keys = depends + build_depends + package.replaces + package.conflicts
+            for os_version in self.distros:
+                resolve_dependencies(unresolved_keys, self.os_name,
+                                     os_version, self.rosdistro,
+                                     peer_packages, fallback_resolver=missing_dep_resolver)
+
+        info("All keys are " + ansi('greenf') + "OK" + ansi('reset') + "\n")
+
+        for package in self.packages.values():
+            if not package.licenses or not package.licenses[0]:
+                error("No license set for package '{0}', aborting.".format(package.name), exit=True)
+
     def pre_branch(self, destination, source):
         if destination in self.rpm_branches:
             return
